/*
 * This file is part of SionTower.
 *
 *
 * David Saltares Márquez (C) 2011
 * <david.saltares@gmail.com>
 *
 *
 * SionTower examples are free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License ad
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) ant later version.
 *
 * SionTower examples are distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SionTower examples.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <iostream>

#include "player.h"
#include "shape.h"
#include "body.h"
#include "collisionManager.h"

/**
 *  @file player.cpp
 *  @author David Saltares Márquez
 *  @date 4-02-2011
 */

using std::cout;
using std::endl;

Player::Player(Ogre::SceneManager* sceneManager,
               Ogre::Camera* camera,
               OIS::Keyboard* keyboard): GameObject(sceneManager, "playerEntity", "personaje.mesh", "playerNode"),
                                         _camera(camera),
                                         _keyboard(keyboard) {
    cout << "Player::Player()" << endl;

    // Creamos el body
    _shapes.push_back(new OrientedBox("playerShape", Ogre::Vector3(0, 0, 0), Ogre::Vector3(0.22 , 0.8, 0.15), Ogre::Matrix3::IDENTITY));
    //_shapes.push_back(new AxisAlignedBox("playerShape", Ogre::Vector3(-0.22,-0.8 ,-0.15), Ogre::Vector3(0.22, 0.8, 0.15)));
    _body = new Body(this, _shapes);
    _body->setType(3);

    _node->setPosition(0, 0.8, 0);

    _entity->setMaterialName("Personaje/TEXFACE/personaje-text.png");

    // Añadimos el body al CollisionManager
    CollisionManager::getSingleton().addBody(_body);

    // Propiedades
    _velocity = Ogre::Vector3(4, 4, 4);
    _actualAngle = Ogre::Degree(0);
    _targetAngle = Ogre::Degree(0);
    _turning = false;
    _walking = false;
    _targetDirection = Ogre::Vector3(0, 0, 1);
    _actualDirection = Ogre::Vector3(0, 0, 1);

    // Sincronizamos la parte visual con la física
    synchronizeBody();
}

Player::~Player() {
    cout << "Player::~Player()" << endl;
}

void Player::update(Ogre::Real deltaT) {

    // Tomamos la dirección de la cámara
    Ogre::Vector3 forward = _camera->getDirection();
    forward.y = 0;

    // Vector relativo a la cámara, ortogonal hacia su right
    Ogre::Vector3 right(-forward.z, 0, forward.x);

    // Calculamos la dirección objetivo en función de la pulsación de las
    // teclas.
    if (_keyboard->isKeyDown(OIS::KC_W) && _keyboard->isKeyDown(OIS::KC_D)) {
        _targetDirection = forward + right;
        _walking = true;
    }
    else if (_keyboard->isKeyDown(OIS::KC_W) && _keyboard->isKeyDown(OIS::KC_A)) {
        _targetDirection = forward - right;
        _walking = true;
    }
    else if (_keyboard->isKeyDown(OIS::KC_S) && _keyboard->isKeyDown(OIS::KC_D)) {
        _targetDirection = -forward + right;
        _walking = true;
    }
    else if (_keyboard->isKeyDown(OIS::KC_S) && _keyboard->isKeyDown(OIS::KC_A)) {
        _targetDirection = -forward - right;
        _walking = true;
    }
    else if (_keyboard->isKeyDown(OIS::KC_W)) {
        _targetDirection = forward;
        _walking = true;
    }
    else if (_keyboard->isKeyDown(OIS::KC_S)) {
        _targetDirection = -forward;
        _walking = true;
    }
    else if (_keyboard->isKeyDown(OIS::KC_D)) {
        _targetDirection = right;
        _walking = true;
    }
    else if (_keyboard->isKeyDown(OIS::KC_A)){
        _targetDirection = -right;
        _walking = true;
    }
    else
        _walking = false;

    // Normalizamos el vector direccion
    _targetDirection.normalise();

    // Si debemos desplazarnos, aplicamos la traslación y calculamos la
    // rotación a aplicar
    if (_walking) {
        // OldPos
        _oldPos = _node->getPosition();

        _node->translate(_velocity * _targetDirection * deltaT, Ogre::Node::TS_WORLD);
        if (_targetDirection != _actualDirection) {
            /*// Si estábamos girando, terminamos la rotación
            if (_turning)
                _node->yaw(_targetAngle);

            // Obtenemos la rotación necesaria
            Ogre::Quaternion rotation = _actualDirection.getRotationTo(_targetDirection,
                                                                       Ogre::Vector3(0, 1, 0));

            _targetAngle = rotation.getYaw();
            _actualAngle = Ogre::Degree(0);
            _turning = true;
            _actualDirection = _targetDirection;*/

            Ogre::Quaternion rotation = _actualDirection.getRotationTo(_targetDirection,
                                                                       Ogre::Vector3(0, 1, 0));

            Ogre::Quaternion actualRotation = _node->getOrientation();

            _node->setOrientation(actualRotation * rotation);

            _actualDirection = _targetDirection;
        }
    }

    /*if (_turning) {
        if (std::abs(_targetAngle.valueDegrees()) > 5) {
            // Calculamos el angulo parcial a girar y lo aplicamos
            _actualAngle = (_targetAngle - _actualAngle) * deltaT * 10;
            _targetAngle -= _actualAngle;
            _node->yaw(_actualAngle);
        } else {
            // Terminamos de girar
            _node->yaw(_targetAngle);
            _turning = false;
        }
    }*/

    synchronizeBody();
}

void Player::restoreOldPosition() {
    cout << "Posición antes: " << getPosition() << endl;
    setPosition(_oldPos);
    cout << "Posición después " << getPosition() << endl;
}

const Ogre::Vector3 Player::getOldPosition() const {
    return _oldPos;
}
